# This file is part of Franklin.
# 
# Franklin is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Franklin is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Franklin.  If not, see <https://www.gnu.org/licenses/>.

import re
import requests
import time
import functools
import logging

import bibtexparser

from .exceptions import DOIError, PDFNotFoundError, BibtexParseError
from .publishers import get_publisher


log = logging.getLogger(__name__)


class Article():
    """A publish research article."""
    _doi_resolution = None
    
    def __init__(self, doi=''):
        self.doi = doi
    
    def url(self):
        """Retrieve the actual URL given the DOI."""
        response = requests.get('https://doi.org/api/handles/{doi}'.format(doi=self.doi)).json()
        response_code = response['responseCode']
        if response_code == 1:
            url = response['values'][0]['data']['value']
        elif response_code == 100:
            raise DOIError("DOI not found: {}".format(response['handle']))
        else:
            raise DOIError("Unexpected DOI error {}".format(response))
        return url
    
    @functools.lru_cache()
    def _bibtex(self):
        """Load the raw bibtex from DOI server."""
        headers = {
            'Accept': 'application/x-bibtex',
        }
        bibtex = requests.get('https://dx.doi.org/{doi}'.format(doi=self.doi), headers=headers)
        return bibtex.text
    
    def metadata(self):
        """Retrieve metadata about this article and return as a dictionary."""
        bibtex = self._bibtex()
        try:
            bibdb = bibtexparser.loads(bibtex)
        except:
            msg = "Could not parse bibtex entry: '{}'".format(bibtex)
            raise BibtexParseError(msg) from None
        if len(bibdb.entries) != 1:
            msg = "Found {} bibtex entries for DOI: '{}'".format(len(bibdb.entries), self.doi)
            raise DOIError(msg)
        metadata = bibdb.entries[0]
        del metadata['ID']
        return metadata
    
    def bibtex(self, id=None):
        """Prepare bibtex entry for this article.
        
        Parameters
        ==========
        id : str
          The entry ID to use. If omitted, ``self.default_id()`` will
          be used.
        
        Returns
        =======
        bibtex : str
          The prepared bibtex entry.
        
        """
        metadata = self.metadata()
        metadata['ID'] = id if id is not None else self.default_id()
        # Convert to bibtex
        db = bibtexparser.bibdatabase.BibDatabase()
        db.entries = [metadata]
        bibtex = bibtexparser.dumps(db)
        return bibtex
    
    def download_pdf(self, fp):
        """Retrieve the PDF for the given article resource.
        
        Parameters
        ==========
        fp : File-like object
          Will receive the PDF contents. Must be writable and in a
          binary mode.
        
        """
        metadata = self.metadata()
        get_pdf = get_publisher(metadata['publisher'])
        pdf_response = get_pdf(doi=self.doi, url=self.url())
        # Save the PDF
        fp.write(pdf_response)
    
    def authors(self):
        metadata = self.metadata()
        authors = metadata['author'].split(' and ')
        return authors
    
    def default_id(self):
        """Prepare a default ID suitable for bibtex."""
        lead_author = self.authors()[-1]
        last_name = lead_author.split(' ')[-1].lower()
        default_id = '{last_name}{year}'.format(last_name=last_name,
                                                year=self.metadata()['year'])
        return default_id
        



import os
import logging
import re
import io
import argparse
from functools import lru_cache

import pandas as pd
from html.parser import HTMLParser
import requests
import bibtexparser
import tqdm
from titlecase import titlecase as titlecase_

from . import exceptions


log = logging.getLogger(__name__)


local_abbreviations = {
    'journal of small papers': 'J. Sm. Papers',
    'materials today nano': 'Mater. Today Nano',
    'materials today': 'Mater. Today',
    'advanced energy materials': 'Adv. Energy Mater.',
    'advanced materials': 'Adv. Mater.',
    'journal of applied physics': 'J. Appl. Phys.',
    'journal of materials chemistry a': 'J. Mater. Chem. A',
    'chemical communications': 'Chem. Commun.',
    'ieee transactions on systems, man, and cybernetics': 'IEEE Transactions on Systems, Man, and Cybernetics',
    'ieee computer graphics and applications': 'IEEE Computer Graphics and Applications',
    'ieee signal processing letters': 'IEEE Signal Processing Letters',
    'ieee transactions on image processing': 'IEEE Transactions on Image Processing',
    'ieee signal processing magazine': 'IEEE Signal Processing Magazine',
    'ieee transactions on pattern analysis and machine intelligence': 'IEEE Transactions on Pattern Analysis and Machine Intelligence',
    'ieee transactions on visualization and computer graphics': 'IEEE Transactions on Visualization and Computer Graphics',
    'analytical chemistry': 'Anal. Chem.',
    'science': 'Science',
    'nature': 'Nature',
    'advanced functional materials': 'Adv. Funct. Mater.',
    'electrochemistry': 'Electrochemistry',
    'advanced materials interfaces': 'Adv. Mater. Interfaces',
    'chemical reviews': 'Chem. Rev.',
    'journal of energy storage': 'J. Energy Storage',
    'atomic data and nuclear data tables': 'At. Data Nucl. Data Tables',
    'radiation physics and chemistry': 'Radiat. Phys. Chem.',
    'materials characterization': 'Mater. Charact.',
    'inorganic chemistry': 'Inorg. Chem.',
    'journal of physics d: applied physics': 'J. Phys. D',
    'frontiers in energy research': 'Front. Energy Res.',
    'journal of physics: condensed matter': 'J. Phys. Condens. Matter',
    'energy storage materials': 'Energy Storage Mater.',
    'spie newsroom': 'SPIE Newsroom',
    'journal of polymer science, part b: polymer physics': 'J. Polym. Sci., Part B: Polym. Phys.',
    'environmental science & technology': 'Environ. Sci. Technol.',
    'journal of electronic imaging': 'J. Electron. Imaging',
}


class CassiAbbreviation():
    span_re = re.compile('<span style="background-color:#7FFFD4">([^<]*)</span>')
    whitespace_re = re.compile(r'\s+')
    class SearchParser(HTMLParser):
        journals = ... # Set in __init__
        coden_re = re.compile('([A-Z]+)_TVALUE')
        title_re = re.compile('<a href="publication.jsp')
        _current_coden = None
        _current_block = None
        _current_journal = None
        TITLE = 'title'
        CODEN = 'coden'
        ABBREVIATION = 'abbreviation'
        
        def __init__(self, *args, **kwargs):
            self.journals = []
            super().__init__(*args, **kwargs)
        
        def _switch_blocks(self, new_block):
            if new_block == self.TITLE:
                self._current_journal = ''
            self._current_block = new_block
        
        def handle_starttag(self, tag, attrs):
            attrs = {key: val for key, val in attrs}
            # Look for a valid CODEN
            coden_match = self.coden_re.search(attrs.get('id', ''))
            if coden_match:
                self._current_coden = coden_match.group(1)
                self._switch_blocks(self.CODEN)
            # We're inside the title section
            elif tag == 'a' and 'publication.jsp' in attrs.get('href', ''):
                self._switch_blocks(self.TITLE)
            # We're inside the abbreviation section
            elif tag == 'td' and attrs.get('class', '') == 'valueAbbr':
                self._switch_blocks(self.ABBREVIATION)
        
        def handle_endtag(self, tag):
            if tag == 'tr':
                self._switch_blocks(None)
        
        def handle_data(self, data):
            if self._current_block == self.TITLE and data.strip():
                self._current_journal += data.strip()
            elif self._current_block == self.ABBREVIATION and data.strip():
                # We found the abbreviation, so now reset everything
                abbr = data.strip()
                new_journal = {
                    'title': self._current_journal,
                    'abbreviation': abbr,
                    'coden': self._current_coden,
                }
                self.journals.append(new_journal)
                self._current_journal = None
                self._current_coden = None
    
    def parse_multiple_sources(self, html_response, journal):
        log.debug("Parsing CASSI response for mutliple results.")
        # Sort through the HTML
        parser = self.SearchParser()
        parser.feed(html_response)
        # See if the requested journal is in the parsed data
        matched_journals = [j for j in parser.journals if j['title'].lower() == journal.lower()]
        if len(matched_journals) == 0:
            raise KeyError("Could not find CASSI entry for '{journal}'.".format(journal=journal))
        if len(matched_journals) > 1:
            print(parser.journals)
            raise exceptions.CassiError("Found {num} CASSI entries for "
                                        "'{journal}'.".format(num=len(matched_journals), journal=journal))
        else:
            abbr = matched_journals[0]['abbreviation']
        return abbr
    
    def parse_single_source(self, html_response, journal):
        log.debug("Parsing CASSI response for single result.")
        # Search the return response for the abbreviated title
        r_str = ('<tr><td class="name">Abbreviated Title</td>'
                 '<td class="value">(?:<span [^>]+>)?'
                 '([-_:A-Za-z0-9. ]+)'
                 '(?:</span>)?</td></tr>')
        match = re.search(r_str, html_response)
        if match:
            abbr = match.group(1)
        else:
            raise exceptions.CassiError("Could not parse single hit for '{journal}'".format(journal=journal))
        return abbr
    
    @lru_cache()
    def __getitem__(self, journal):
        """Retrieve abbreviated journal name from CASSI."""
        # Ask for a validation code for having accepted the terms of service
        cookies = {'UserAccepted': 'YES'}
        response = requests.get('https://cassi.cas.org/search.jsp', cookies=cookies)
        content = str(response.content)
        if 'You have to enable JavaScript' in content:
            raise exceptions.CASSIError("Could not accept CASSI terms.")
        # Extract the validation code from the response
        r_str = '<input type="hidden" name="c" value="([^"]+)"'
        match = re.search(r_str, content)
        if match:
            c_code = match.group(1)
        else:
            raise exceptions.CASSIError("Could not extract CASSI terms validation code.")
        # Perform the actual search
        post_data = {'searchIn': 'titles',
                     'searchFor': journal,
                     'c': c_code}
        if '&' not in journal:
            post_data['exactMatch'] = 'on'
        response = requests.post('https://cassi.cas.org/searching.jsp',
                                 data=post_data)
        # Strip out the background highlighting and extra whitespace
        response_text = self.span_re.sub(r'\1', response.text)
        response_text = self.whitespace_re.sub(r' ', response_text)
        # Deal with multiple similar database hits
        if "Results of Search for" in response_text:
            abbr = self.parse_multiple_sources(response_text, journal)
        else:
            abbr = self.parse_single_source(response_text, journal)
        log.info('Abbreviated "{journal}" to "{abbr}" by CASSI'.format(journal=journal, abbr=abbr))
        return abbr


@lru_cache()
def abbreviate_journal(journal, use_native, use_cassi, use_ltwa):
    # Build a list of which databases to query
    abbr_dbs = []
    if use_native:
        abbr_dbs.append(local_abbreviations)
    if use_cassi:
        abbr_dbs.append(CassiAbbreviation())
    if use_ltwa:
        abbr_dbs.append(LTWAAbbreviation())
    # Clean up the journal title a little
    journal = journal.strip()
    if journal.lower()[0:3] == 'the':
        journal = journal[4:]
        journal = journal.replace(r'\&', '&').replace('{', '').replace('}', '')
        journal = journal.replace(r'\ ', ' ').replace(r'\n', ' ')
        # Do the lookup in each database
    for abbrs in abbr_dbs:
        try:
            new_journal = abbrs[journal.lower()]
        except KeyError:
            continue
        except (exceptions.CassiError, requests.exceptions.ConnectionError) as e:
            log.warning(e)
        else:
            break
    else:
        log.warning("Could not abbreviate journal '{journal}'.".format(journal=journal))
        new_journal = journal
    return new_journal


def abbreviate_bibtex_journals(bibfile: str, output: str=None,
                               fix_titlecase=True, use_native=True,
                               use_cassi=True, use_ltwa=True):
    """Parse a bibtex file and abbreviate journal titles.
    
    Parameters
    ==========
    bibfile
      The input bibtex file (open file object) to be parsed.
    output
      The open bibtex file object to receive the parsed bibtex
      content.
    fix_titlecase
      Convert article title to proper title-case.
    use_native
      Use the built-in database of journal abbreviations.
    use_cassi
      Use the ACS CASSI database of journal abbreviations.
    use_ltwa
      Use the ISSN list of title word abbreviations (LTWA).
    
    """
    olddb = bibtexparser.load(bibfile)
    newdb = bibtexparser.bibdatabase.BibDatabase()
    for entry in tqdm.tqdm(olddb.entries):
        # Fix any entries with double curly braces
        fix_curly_braces(entry)
        # Abbreviate journal titles
        if 'journal' in entry.keys():
            entry['journal'] = abbreviate_journal(entry['journal'], use_native=use_native,
                                                  use_cassi=use_cassi, use_ltwa=use_ltwa)
        # Fix the title-case of the journal title
        title_keys = ['title', 'booktitle']
        if fix_titlecase:
            for title_key in title_keys:
                if title_key in entry.keys():
                    entry[title_key] = titlecase(entry[title_key])
        newdb.entries.append(entry)
    # Save the updated bibtext database to the new file
    bibtexparser.dump(newdb, output)


def abbreviate_journals_cli(argv=None):
    # Parse the arguments
    parser = argparse.ArgumentParser(description='Abbreviate journal titles in a Bibtex file.')
    parser.add_argument('bibfile', help='bibtex input file')
    parser.add_argument('-d', '--debug', action='store_true', help='Very verbose logging output')
    parser.add_argument('-v', '--verbose', action='store_true', help='Verbose logging output')
    parser.add_argument('-q', '--quiet', action='store_true', help='Suppress logging output')
    parser.add_argument('-o', '--output', help='bibtex input file')
    parser.add_argument('-f', '--force', help='Overwrite existing output file.',
                        action='store_true')
    parser.add_argument('-t', '--skip-titlecase', dest='fix_titlecase', action='store_false',
                        help="Don't convert article titles to proper title case.",)
    parser.add_argument('-n', '--no-native', dest='use_native', action='store_false',
                        help='Do not query the native override database.')
    parser.add_argument('-c', '--no-cassi', dest='use_cassi', action='store_false',
                        help='Do not query the CASSI database.')
    parser.add_argument('-l', '--no-ltwa', dest='use_ltwa', action='store_false',
                        help='Do not abbreviate by LTWA.')
    parser.add_argument('--logfile', help='file to receive the debug log')
    args = parser.parse_args(argv)
    # Prepare logging
    if args.debug:
        loglevel = logging.DEBUG
    elif args.verbose:
        loglevel = logging.INFO
    elif args.quiet:
        loglevel = logging.CRITICAL
    else:
        loglevel = logging.WARNING
    logging.basicConfig(filename=args.logfile, level=loglevel)
    # Get a default output filename if necessary
    output = args.output
    if output is None:
        base, ext = os.path.splitext(args.bibfile)
        output = '{base}-abbrev{ext}'.format(base=base, ext=ext)
    # Check if the file exists
    if os.path.exists(output) and not args.force:
        raise exceptions.FileExistsError(
            "Output file '{output}' already exists. Use `--force` to overwrite.".format(output=output))
    # Call the actual function
    with open(args.bibfile, mode='r') as bibfile, open(output, mode='w') as output:
        abbreviate_bibtex_journals(bibfile=bibfile, output=output,
                                   fix_titlecase=args.fix_titlecase,
                                   use_native=args.use_native,
                                   use_cassi=args.use_cassi,
                                   use_ltwa=args.use_ltwa)


class LTWAAbbreviation():
    ltwa_url = 'https://www.issn.org/wp-content/uploads/2013/09/LTWA_20160915.txt'
    
    @lru_cache()
    def ltwa_list(self):
        fp = io.StringIO(requests.get(self.ltwa_url).text)
        df = pd.read_csv(fp, delimiter='\t')
        return df
    
    def find_abbrev_in_df(self, word, df):
        for idx, (word_re, curr_abbrev, lang) in df.iterrows():
            word_re = word_re.replace('-', '(.*)')
            match = re.match(word_re, word.lower())
            # We found the matching entry, so stop looking
            if match:
                abbrev = curr_abbrev if curr_abbrev != 'n.a.' else word
                group_idx = 1
                while '-' in abbrev:
                    abbrev = abbrev.replace('-', match.group(group_idx), 1)
                    group_idx += 1
                break
        else:
            abbrev = word
        return abbrev
    
    def __getitem__(self, title):
        ignored_words = ['of', 'the', 'a', '&', 'and']
        df = self.ltwa_list()
        abbreviations = []
        for word in title.split():
            if word not in ignored_words:
                abbreviations.append(self.find_abbrev_in_df(word, df))
        # Convert the list of matched abbreviations/words to a new journal title
        new_title = ' '.join(abbreviations)
        # Convert to title case (except for initialisms)
        new_title = new_title.title()
        acronyms = [s for s in abbreviations if s.isupper()]
        for acronym in acronyms:
            new_title = new_title.replace(acronym.title(), acronym)
        log.info('Abbreviated "{title}" to "{new_title}" by LTWA'.format(title=title, new_title=new_title))
        return new_title


def _tex_callback(word, **kwargs):
    # only lower case words get fixed
    if not word.islower():
        return word


def titlecase(title):
    new_title = title.replace('\n', ' ')
    new_title = titlecase_(new_title, callback=_tex_callback)
    return new_title


curly_braces_re = re.compile(
    '\{' # The starting (bad) curly brace...
    '(?![^\{\}]*\}.+)' # ...but only if it's not part of a real curly group
    '.*' # Regular title characters
    '\}$' # The losing (bad) curly brace...
)


def fix_curly_braces(entry):
    """Convert bibtex entries with extra curly braces.
    
    > @article{myentry,
    >   title = {{My title}},
    > }
    becomes...
    > @article{myentry,
    >   title = {My title},
    > }
    
    """
    for key, val in entry.items():
        match = curly_braces_re.match(val)
        if match:
            print(val)
            entry[key] = val[1:-1]




# This file is part of Franklin.
# 
# Franklin is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Franklin is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Franklin.  If not, see <https://www.gnu.org/licenses/>.

import os
import argparse
import re
import logging
from typing import List, Iterable

import bibtexparser

from .article import Article
from .version import __version__
from . import exceptions

log = logging.getLogger(__name__)


def parse_doi(doi):
    """Validate and extract a digital object identifier."""
    regex = ('(https?://)?((dx\.)?doi.org/)?'
             '([0-9.]+/[-a-zA-Z0-9._:;()/#+<>]+)')
    match = re.match(regex, doi)
    if match:
        new_doi = match.group(4)
        logging.debug("Parsed DOI '%s' to '%s'.", doi, new_doi)
    else:
        raise exceptions.DOIError("Malformed DOI: '%s'" % doi)
    return new_doi


def existing_ids(doi: str, bib_entries: Iterable) -> List:
    """Look through a bibtex bibliography and check for duplicate entry.
    
    Parameters
    ==========
    doi
      The digital object identifier to search for.
    bib_entries
      The bibliography data to look in.
    
    Returns
    =======
    existing_ids : 
      List of the IDs of existing entries with this same DOI.
    
    """
    existing_ids = [e.get('ID') for e in bib_entries if e.get('doi', '').lower() == doi.lower()]
    if len(existing_ids) == 0:
        existing_ids = None
    return existing_ids


def validate_bibtex_id(base_id, pdfs, bibtex_entries):
    """Find a unique identifier for this bibtex entry and PDF file.
    
    Parameters
    ==========
    base_id : str
      A starting point for this ID. If it's already taken, 'base_id-2'
      will be tried, then 'base_id-3', etc.
    pdfs : iterable
      Filenames of existing PDF's, for example from
    ``os.path.listdir()`` to be check for existing IDs.
    bibtex : iterable
      Bibtex entries (dicts) to be checked for a unique ID.
    
    Returns
    =======
    new_id : str
      The new, unique identifier for this entry.
    
    """
    # Helper function
    def id_is_taken(this_id, pdf_ids, bibtext_ids):
        return (this_id in pdf_ids or this_id in bibtex_ids)
    # Prepare lists of existing IDs
    stripped_pdfs = [os.path.splitext(pdf)[0] for pdf in pdfs]
    bibtex_ids = [entry['ID'] for entry in bibtex_entries]
    # Iterate through all the indices and check for available IDs
    idx = 2
    new_id = base_id
    while id_is_taken(new_id, stripped_pdfs, bibtex_ids):
        new_id = "{}-{}".format(base_id, idx)
        idx += 1
    return new_id


def add_bibtex_entry(bibtex, bibtexfile):
    """Add some metadata to an open bibtex file.
    
    Parameters
    ==========
    bibtex : dict
      The bibtex string that will be added to the new bibtex entry.
    bibtexfile : file-like object
      An open, writable, text-mode file (ideally, ``mode='a+'``). The
      new bibtex entry will be added to the end.
    
    """
    # Save the new tex in the bibtex file
    bibtexfile.seek(0, os.SEEK_END)
    bibtexfile.write('\n')
    bibtexfile.write(bibtex)


def fetch_doi(doi, bibfile, pdf_dir, bibtex_id=None, retrieve_pdf=True):
    """Retrieve a document by its Digital object idetifier.
    
    This function will retrieve metadata and optionally the PDF for
    the requested document. A new entry will be added to the bibtex
    file with a unique ID and, if requested, the PDF will be added to
    the ``pdf_dir`` with a filename matching the new ID.
    
    Parameters
    ==========
    doi : str
      The digital object identifier for the object.
    bibfile : File-like object
      An open, writable (``mode='a+'``), text-mode file that will
      receive the new bibtex entry.
    pdf_dir : str
      Directory in which to put the PDF.
    bibtex_id : str
      The requested bibtex ID for this document. Will not necessarily
      be the one that is used if that ID is already taken.
    retrieve_pdf : bool
      Whether to attempt to download the PDF of the document (default,
      True).
    
    Returns
    =======
    new_id : str
      The bibtex ID for the new entry.
    
    """
    # Read in the existing bibtex entries
    bibfile.seek(0)
    bibdb = bibtexparser.loads(bibfile.read() + ' ')
    # Create the article class
    article = Article(doi=doi)
    # Retrieve the article metdata
    metadata = article.metadata()
    # Check if the entry already exists in the refs file
    _existing_ids = existing_ids(doi=doi, bib_entries=bibdb.entries)
    if _existing_ids:
        raise exceptions.DuplicateDOIError(
            "Existing entries found for DOI '{}': {}".format(doi, _existing_ids))
    # Determine a unique ID for this entry/PDF
    default_id = bibtex_id if bibtex_id is not None else article.default_id()
    new_id = validate_bibtex_id(base_id=default_id,
                                pdfs=os.listdir(pdf_dir),
                                bibtex_entries=bibdb.entries)
    # Download the PDF
    if retrieve_pdf:
        pdffile = os.path.join(pdf_dir, '{}.pdf'.format(new_id))
        try:
            pdffp = open(pdffile, 'wb')
            article.download_pdf(fp=pdffp)
        except:
            # Delete the file if an exception occurred
            pdffp.close()
            os.remove(pdffile)
            raise
        else:
            # Upon successful download, just close the file
            pdffp.close()
    # Add the bibtex entry to the bibfile
    bibtex = article.bibtex(id=new_id)
    add_bibtex_entry(bibtex, bibfile)
    return new_id


def main(argv=None):
    # Prepare the command line arguments
    parser = argparse.ArgumentParser(
        description='Fetch an article by its digital object identifier'
    )
    parser.add_argument('doi', type=str,
                        help='the digital object identifier to retrieve')
    parser.add_argument('-p', '--pdf-dir', dest='pdf_dir', default='./papers/',
                        metavar='PATH',
                        help='where to store the downloaded PDF')
    parser.add_argument('-b', '--bibtex-file', dest='bibfile', default='./refs.bib',
                        metavar='FILE',
                        help='will add the new bibtex entry to this file')
    parser.add_argument('-i', '--bibtex-id', dest='bibtex_id', default=None,
                        help="a default bibtex id, but may be modified if it already exists")
    parser.add_argument('--no-pdf', dest='retrieve_pdf', action='store_false',
                        help="don't attempt to download the article as a PDF")
    parser.add_argument('-d', '--debug', dest='debug', action='store_true',
                        help="show detailed debug information via the logging platform")
    parser.add_argument('-f', '--force', dest='force', action='store_true',
                        help="force creation of files, directories, etc.")
    parser.add_argument('-V', '--version', action='version', version='%(prog)s v{}'.format(__version__))
    # Parse the command line arguments
    args = parser.parse_args(argv)
    # Start logging
    debug = args.debug
    if debug:
        level = logging.DEBUG
    else:
        level = logging.WARNING
    logging.basicConfig(level=level)
    doi = parse_doi(args.doi)
    force = args.force
    bibfile = args.bibfile
    bibtex_id = args.bibtex_id
    pdf_dir = args.pdf_dir
    retrieve_pdf = args.retrieve_pdf
    # Check if the file exists
    if not os.path.exists(bibfile) and not force:
        raise exceptions.BibtexFileNotFoundError("Cannot find bibtex file: {}".format(bibfile))
    # Check if the PDF directory exists
    if not os.path.exists(pdf_dir) and retrieve_pdf:
        if force:
            os.makedirs(pdf_dir)
        else:
            raise exceptions.BibtexFileNotFoundError("Cannot find PDF folder: {}".format(pdf_dir))
    # Do the actual DOI fetching
    logging.debug("Opening bibfile '%s'", bibfile) 
    with open(bibfile, mode='a+') as bibfp:
        new_id = fetch_doi(doi=doi, bibfile=bibfp, pdf_dir=pdf_dir,
                           bibtex_id=bibtex_id, retrieve_pdf=retrieve_pdf)
    # Confirm successful retrieval
    msg = "Saved entry as {} ({}.pdf)".format(new_id, os.path.join(pdf_dir, new_id))
    log.info(msg)
    print("Saved entry as {} ({}.pdf)".format(new_id, os.path.join(pdf_dir, new_id)))


if __name__ == '__main__':
    main()



class DOIError(RuntimeError):
    """Resolution of a digital object identifier has encountered an error."""
    pass


class DuplicateDOIError(RuntimeError):
    """Trying to add a new bibtex entry when the file already contains this DOI."""
    pass


class BibtexFileNotFoundError(FileNotFoundError):
    pass


class BibtexParseError(RuntimeError):
    """The bibtex string could not be parsed properly."""
    pass


class CassiError(RuntimeError):
    pass


class PDFNotFoundError(FileNotFoundError):
    pass


class FileExistsError(RuntimeError):
    pass


class ConfigError(RuntimeError):
    pass


class UnknownPublisherError(KeyError):
    pass




import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

# Read the project version from franklin/version.py
version = {}
with open("franklin/version.py") as fp:
    exec(fp.read(), version)

setup(
    name = "franklin",
    version = version['__version__'],
    author = "Mark Wolfman",
    author_email = "canismarko@gmail.com",
    description = "Retrieve scientific papers and manage associated citations.",
    license = "GPLv3",
    keywords = "",
    url = "https://github.com/canismarko/franklin",
    packages=['franklin'],
    long_description=read('README.rst'),
    install_requires=[
        'bibtexparser', 'requests', 'tqdm', 'pandas', 'titlecase',
    ],
    entry_points = {
        'console_scripts': [
            'fetch-doi=franklin.fetch_doi:main',
            'abbreviate-journals=franklin.journals:abbreviate_journals_cli',
        ],
    },
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
)
